@@ -8,7 +8,7 @@ from ss2hcsp.hcsp import expr
 from ss2hcsp.hcsp.hcsp import HCSP, Function
 from ss2hcsp.hcsp.parser import parse_expr_with_meta, parse_expr_with_meta, parse_hp_with_meta, \
     parse_hoare_triple_with_meta
-from hhlpy.hhlpy_without_dG import CmdVerifier
+from hhlpy.hhlpy import CmdVerifier
 from hhlpy.wolframengine_wrapper import found_wolfram, session
 
 def runFile(self, file, 
@@ -45,13 +45,9 @@ def runFile(self, file,
             print("%s:" % str(pos))
             for vc in vcs:
                 print(vc.expr, 
-                "pos:", vc.pos, 
-                "annot_pos:", vc.annot_pos,
                 "categ:", vc.categ,
                 "blabel:", str(vc.blabel),
-                "comp_label:", str(vc.comp_label),
-                "vc:", vc.vc,
-                "pc:", vc.pc)
+                "comp_label:", str(vc.comp_label))
 
     # Use SMT to verify all verification conditions
     self.assertTrue(verifier.verify())
@@ -66,6 +62,7 @@ def runFile(self, file,
     if expected_vcs:
         for pos, vcs in expected_vcs.items():
             vcs = [parse_expr_with_meta(vc) for vc in vcs]
+            assert pos in verifier.infos, "pos {} is not in infos.".format(pos)
             actual_vcs = [vc.expr for vc in verifier.infos[pos].vcs if not is_trivial(vc.expr)]
             self.assertEqual(set(vcs), set(actual_vcs), 
             "\nExpect: {}\nActual: {}".format([str(vc) for vc in vcs],[str(vc) for vc in actual_vcs]))
@@ -88,7 +85,7 @@ class HHLPyTest(unittest.TestCase):
             x := x + 1;
             post [x >= 1];
         """)
-        self.assertEqual(str(res.pre[0]), "x >= 0")
+        self.assertEqual(str(res.pre[0].expr), "x >= 0")
         self.assertEqual(str(res.post[0].expr), "x >= 1")
         self.assertEqual(str(res.hp), "x := x + 1;")
 
@@ -167,7 +164,7 @@ class HHLPyTest(unittest.TestCase):
 
     def testVerify5(self):
         runFile(self, file="test5.hhl", print_vcs=False,
-                  expected_vcs={((), (0,)): ["x >= 0 -> x + 1 >= 0"]})
+                  expected_vcs={((0,), ()): ["x >= 0 -> x + 1 >= 0"]})
 
     def testVerify5_1(self):
         runFile(self, file="test5_1.hhl",
@@ -187,7 +184,7 @@ class HHLPyTest(unittest.TestCase):
     # TODO: 
     # def testVerify7_1(self):
     #     # {true} <x_dot = 2 & x < 10> {true}
-    #     runVerify(self, pre="true", hp="<x_dot = 2 & x < 10>", post="true", print_vcs=True)
+    #     runVerify(self, pre="true", hp="<x_dot = 2 & x < 10>", post="true", print_vcs=False)
 
     def testVerify7_2(self):
         runFile(self, file="test7_2.hhl",
@@ -199,15 +196,15 @@ class HHLPyTest(unittest.TestCase):
 
     def testVerify11(self):
         runFile(self, file="test11.hhl",
-                  expected_vcs={((), ()): ["x0 >= 0 -> x0 >= 0 -> x1 >= 1 -> x1 >= 1"]})
+                  expected_vcs={((), ()): ["x1 >= 0 -> x1 >= 0 -> x2 >= 1 -> x2 >= 1"]})
 
     def testVerify12(self):
         runFile(self, file="test12.hhl",
-                  expected_vcs={((), ()): ["x >= 0 -> y0 >= x + 1 -> y0 >= 1"]})
+                  expected_vcs={((), ()): ["x >= 0 -> y1 >= x + 1 -> y1 >= 1"]})
 
     def testVerify13(self):
         runFile(self, file="test13.hhl",
-                  expected_vcs={((), ()): ["x >= 0 -> y1 >= x + 1 ->y0 >= x + 1 + 1 -> y0 >= 2"]})
+                  expected_vcs={((), ()): ["x >= 0 -> y2 >= x + 1 ->y1 >= x + 1 + 1 -> y1 >= 2"]})
 
     def testVerify9(self):
         runFile(self, file="test9.hhl", print_vcs=False)
@@ -273,15 +270,14 @@ class BasicHHLPyTest(unittest.TestCase):
     def testBasic3(self):
         runFile(self, file="basic3.hhl",
                   expected_vcs={((), ()): ["x >= 0 -> x + 1 >= 1"],
-                                ((1,), (0,)): ["x >= 1 -> x + 1 >= 1"]})
+                                ((1, 0), ()): ["x >= 1 -> x + 1 >= 1"]})
 
     def testBasic4(self):
-        runFile(self, file="basic4.hhl",
-                  print_vcs=False)
+        runFile(self, file="basic4.hhl",)
 
     def testBasic5(self):
         runFile(self, file="basic5.hhl",
-                  expected_vcs={((), ()): ["x >= 0 -> x0 >= 1 -> x0 >= 1"]})
+                  expected_vcs={((), ()): ["x >= 0 -> x1 >= 1 -> x1 >= 1"]})
 
     # TODO: Basic benchmark problem 6 is hard to translate into HCSP program.
 
@@ -292,7 +288,7 @@ class BasicHHLPyTest(unittest.TestCase):
         runFile(self, file="basic8.hhl")
 
     def testBasic9(self):
-        runFile(self, file="basic9.hhl")
+        runFile(self, file="basic9.hhl", print_vcs=False)
     
     def testBasic10(self):
         runFile(self, file="basic10.hhl",
@@ -303,9 +299,9 @@ class BasicHHLPyTest(unittest.TestCase):
 
     def testBasic12(self):
         runFile(self, file="basic12.hhl",
-                  expected_vcs={((), ()): ["y >= 0 -> x >= 0 && y >= 0 -> t0 >= 0 -> t0 > 0 -> y >= 0",
-                                           "y >= 0 -> x >= 0 && y >= 0 -> t0 >= 0 -> t0 > 0 -> x >= 0",
-                                           "y >= 0 -> x >= 0 && y >= 0 -> t0 >= 0 -> t0 <= 0 -> x >= 0"]})
+                  expected_vcs={((), ()): ["y >= 0 -> x >= 0 && y >= 0 -> t1 >= 0 -> t1 > 0 -> y >= 0",
+                                           "y >= 0 -> x >= 0 && y >= 0 -> t1 >= 0 -> t1 > 0 -> x >= 0",
+                                           "y >= 0 -> x >= 0 && y >= 0 -> t1 >= 0 -> t1 <= 0 -> x >= 0"]})
 
     def testBasic13(self):
         runFile(self, file="basic13.hhl",)
@@ -315,19 +311,21 @@ class BasicHHLPyTest(unittest.TestCase):
         runFile(self, file="basic14.hhl",)
 
     def testBasic15(self):
-        runFile(self, file="basic15.hhl")
-                # expected_vcs={((), ()): ["x > 0 -> 0 < 1 -> (\exists y. x * y * y == 1)", \
-                #                          "x > 0 -> 0 >= 1 -> x > 0"],
-                #               ((1,), ()): ["(\exists y. x * y * y == 1) && t == 1 -> x > 0"]})
+        runFile(self, file="basic15.hhl", 
+                expected_vcs={((), ()): ["x > 0 -> t1 >= 0 -> t1 > 0 -> (\exists y. x * y * y == 1)", \
+                                         "x > 0 -> t1 >= 0 -> t1 <= 0 -> x > 0"],
+                              ((1,), ()): ["x * y * y == 1 && t == 0 -> x > 0"],
+                              ((1,), (0,)): ["t > 0 -> x * y * (y / 2) + (x * (y / 2) + -x * y) * y == 0"]})
 
     def testBasic16(self):
         runFile(self, file="basic16.hhl",)
 
     def testBasic17(self):
-        runFile(self, file="basic17.hhl",)
-                #   expected_vcs={((), ()): ["y > 0 -> x > 0 && y > 0 -> t0 >= 0 -> t0 > 0 -> (\exists z. x * z * z == 1)",
-                #                            "y > 0 -> x > 0 && y > 0 -> t0 >= 0 -> t0 <= 0 -> x > 0"],
-                #                 ((1,), ()): ["y > 0 -> (\\exists z. x * z * z == 1) && t == 0 -> x > 0"]})
+        runFile(self, file="basic17.hhl", print_vcs=False,
+                expected_vcs={((), ()): ["y > 0 -> x > 0 && y > 0 -> t1 >= 0 -> t1 > 0 -> (\exists z. x * z * z == 1)",
+                                         "y > 0 -> x > 0 && y > 0 -> t1 >= 0 -> t1 <= 0 -> x > 0"],
+                              ((1,), ()): ["y > 0 -> x * z * z == 1 && t == 0 -> x > 0"],
+                              ((1,), (0,)): ["y > 0 -> t > 0 -> x * z * (y * z / 2) + (x * (y * z / 2) + -y * x * z) * z == 0"]})
 
     def testBasic18(self):
         runFile(self, file="basic18.hhl", print_vcs=False)
@@ -433,7 +431,8 @@ class BasicHHLPyTest(unittest.TestCase):
 
     def testBasic47(self):
         runFile(self, file="basic47.hhl",)
-
+    
+    # TODO: 48 is very slow.(But fast on interface)
     def testBasic48(self):
         runFile(self, file="basic48.hhl",)
 
@@ -523,6 +522,10 @@ class BasicHHLPyTest(unittest.TestCase):
     
     # TODO: Basic benchmark, problem 56 - 60, cannot be written into hcsp program.
 
+    # TODO: Proofs or invariants Unknown
+    # def testOscillator(self):
+    #     runFile(self, file="oscillator.hhl",)
+
 
 class NonlinearHHLPyTest(unittest.TestCase):
     
@@ -599,7 +602,10 @@ class NonlinearHHLPyTest(unittest.TestCase):
     def testNonlinear18(self):
         runFile(self, file="nonlinear18.hhl")
 
-    # TODO: Nonlinear benchmark, problem 19, 20. The ODE rule is not clear.
+    # TODO: Nonlinear benchmark, problem 19. The ODE rule is not clear.
+
+    def testNonlinear20(self):
+        runFile(self, file="nonlinear20.hhl")
 
     def testNonlinear21(self):
         runFile(self, file="nonlinear21.hhl",)
@@ -609,8 +615,9 @@ class NonlinearHHLPyTest(unittest.TestCase):
 
     def testNonlinear23(self):
         runFile(self, file="nonlinear23.hhl")
-                  
-    # TODO: Nonlinear benchmark, problem 24. ODE rule not clear.
+
+    def testNonlinear24(self):
+        runFile(self, file="nonlinear24.hhl")
 
     def testNonlinear25(self):
         runFile(self, file="nonlinear25.hhl")
@@ -704,7 +711,7 @@ class NonlinearHHLPyTest(unittest.TestCase):
     def testNonlinear55(self):
         runFile(self, file="nonlinear55.hhl")
 
-    # TODO: nonlinear problem 56, ODE rule is not clear. Not proved by keymaerax yet.
+    # TODO: nonlinear problem 56, ODE rule is not clear.
     # def testNonlinear56(self):
     #     # Nonlinear benchmark, problem 56
     #     # {-3<=x & x<=-2 & -3<=y & y<=-2}
@@ -821,7 +828,17 @@ class NonlinearHHLPyTest(unittest.TestCase):
 
     # TODO: Nonlinear benchmark, problem 91. Invariants unknown.
 
-    # TODO: Nonlinear benchmark, problem 92, 93, 94, 95. ODE rule unknown.
+    def testNonlinear92(self):
+        runFile(self, file="nonlinear92.hhl")
+
+    def testNonlinear93(self):
+        runFile(self, file="nonlinear93.hhl")
+
+    def testNonlinear94(self):
+        runFile(self, file="nonlinear94.hhl")
+
+    def testNonlinear95(self):
+        runFile(self, file="nonlinear95.hhl")
 
     def testNonlinear96(self):
         runFile(self, file="nonlinear96.hhl")
@@ -936,6 +953,17 @@ class NonlinearHHLPyTest(unittest.TestCase):
 
 class SSHHLPyTest(unittest.TestCase):
 
+    @classmethod
+    def setUpClass(cls):
+        try:
+            session.start()
+        except Exception as e:
+            session.start()
+
+    @classmethod
+    def tearDownClass(cls):
+        session.terminate()
+
     def testBouncing(self):
         runFile(self, file="simulink/sf_bouncing.hhl")
 
@@ -949,6 +977,23 @@ class SSHHLPyTest(unittest.TestCase):
         runFile(self, file="simulink/sl_delay.hhl")
 
 
+class AdvancedHHLPyTest(unittest.TestCase):
+
+    @classmethod
+    def setUpClass(cls):
+        try:
+            session.start()
+        except Exception as e:
+            session.start()
+
+    @classmethod
+    def tearDownClass(cls):
+        session.terminate()
+
+    def testAdvanced1(self):
+        runFile(self, file="advanced1.hhl",)
+
+
 if __name__ == "__main__":
     unittest.main()
 
